# %%
import matplotlib
import matplotlib.colors as colors
import matplotlib.pyplot as plt
import numpy as np
import xarray as xr

# import os
# os.chdir(os.getcwd()+'/phase-tunable-pmm/analysis/')


# %%
tex_settings = {
    "text.usetex": True,
    "font.family": "lmr",
    "font.serif": "Computer Modern Roman",
    "text.latex.preamble": r"\usepackage{siunitx} \usepackage[version=4]{mhchem} \sisetup{detect-all}",
    "font.size": 11,
}
matplotlib.rcParams.update(tex_settings)
cm = 1/2.54
plot_width = 15 * cm

# %%
# Figure 3
data_energies = xr.load_dataset('../data/fig_1_energies.nc')
mus = data_energies.coords[r'$\mu_{LR}$'].data
phis = data_energies.coords[r'$\phi$'].data
mus_M = data_energies.coords[r'$\mu_M$'].data
energies_zero_phase, energies_finite_phase = data_energies['energy'].data
muLR, phi, muLR_z, phi_z = list(data_energies.attrs.values())

# %%
phis

# %%
data_wfs = xr.load_dataset('../data/fig_1_wfs.nc')
majs_z, majs = data_wfs['wfs'].data

# %%
data_transport = xr.load_dataset('../data/fig_1_transport.nc')
mu_dets = data_transport.coords[r'$\delta\mu_L$'].data
biases = data_transport.coords[r'$V_\text{bias}$'].data
total_res = data_transport['conductance'].data
res = data_transport.attrs['res']

# %%
site_labels = [
    r'$L_\uparrow$', r'$L_\downarrow$',
    r'$M_\uparrow$', r'$M_\downarrow$',
    r'$R_\uparrow$', r'$R_\downarrow$'
]

# %%
norm = colors.TwoSlopeNorm(vmin=-0.15, vcenter=0, vmax=0.15)

# %%
fig, axes = plt.subplots(figsize=(plot_width * 0.95, 5), ncols=2, nrows=3)
fig.tight_layout(w_pad=0, h_pad=2)
ax = axes[0, 0]
im = ax.pcolormesh(
    mus, 2 * phis/np.pi, energies_zero_phase,
    cmap='seismic', norm=norm, rasterized=True
)
ax.scatter(muLR, 2*phi/np.pi, c='black', marker='x')
ax.set_ylabel(r'$\phi/\pi$' )
ax.set_xlabel(r'$\mu_{LR}$' )
ax.set_title(r'$\mu_M =$'+str(mus_M[0]))

ax = axes[0, 1]
im = ax.pcolormesh(
    mus, 2 * phis/np.pi, energies_finite_phase,
    cmap='seismic', norm=norm, rasterized=True
)
ax.scatter(muLR_z, 2*phi_z/np.pi, c='black', marker='x')
ax.set_xlabel(r'$\mu_{LR}$' )
ax.set_yticklabels([])
ax.set_title(r'$\mu_M =$'+str(mus_M[1]))

cax = fig.add_axes([ax.get_position().x1+0.01,ax.get_position().y0,0.02,ax.get_position().height])
cbar = plt.colorbar(im, cax=cax) # Similar to fig.colorbar(im, cax = cax)
cbar.set_label(r'$E_{\text{odd}} - E_{\text{even}}$')

ax = axes[1, 0]
ax.bar(np.arange(6) + 0.2, np.abs(majs[0]), width=0.3, color='orange', label=r'$|\gamma_1|$')
ax.bar(np.arange(6) - 0.2 , np.abs(majs[1]), width=0.3, color='purple', label=r'$|\gamma_2|$')
ax.set_xticklabels(site_labels)
ax.set_xticks(range(len(site_labels)))
ax.set_ylim(0, 1)
ax.set_ylabel(r'$|\psi_{i}|^2$')
ax.spines['right'].set_visible(False)
ax.spines['top'].set_visible(False)

ax = axes[1, 1]
ax.bar(np.arange(6) + 0.2, np.abs(majs_z[0]), width=0.3, color='orange', label=r'$\psi_1$')
ax.bar(np.arange(6) - 0.2, np.abs(majs_z[1]), width=0.3, color='purple', label=r'$\psi_2$')
ax.set_xticklabels(site_labels)
ax.set_xticks(range(len(site_labels)))
ax.legend(frameon=False)
ax.set_ylim(0, 1)
ax.set_yticklabels([])
ax.spines['right'].set_visible(False)
ax.spines['top'].set_visible(False)

ax = axes[-1, :]
top = 5e-2# np.max(total_res[:, :, :, 0])
data = total_res[0, :, :, 0]
cond = np.array(np.gradient(data, res, axis=1))
im = ax[0].pcolormesh(mu_dets, biases, cond.T, vmin=0, vmax=top, rasterized=True)
ax[0].set_xlabel(r'$\delta\mu_L$')
ax[0].set_ylabel(r'$V_\text{bias}$')

data = total_res[1, :, :, 0]
cond = np.array(np.gradient(data, res, axis=1))
im = ax[1].pcolormesh(mu_dets, biases, cond.T, vmin=0, vmax=top, rasterized=True)
ax[1].set_xlabel(r'$\delta\mu_L$')
ax[1].set_yticklabels([])

ax = axes[-1, -1]
cax = fig.add_axes([ax.get_position().x1+0.01,ax.get_position().y0,0.02,ax.get_position().height])
cbar = plt.colorbar(im, cax=cax) # Similar to fig.colorbar(im, cax = cax)
cbar.set_label(r'$G_\text{LL}[e^2/h]$')

axes[0, 0].text(-0.6, 2.1, '(a)')
axes[0, 0].text(1.1, 2.1, '(b)')
axes[0, 0].text(-0.6, -.8, '(c)')
axes[0, 0].text(1.1, -.8, '(d)')
axes[0, 0].text(-0.6, -4, '(e)')
axes[0, 0].text(1.1, -4, '(f)')

# %%
plt.savefig(
    "../publication/figures/phase_diagram.svg", bbox_inches='tight', transparent=False, dpi=500
    )

# %%
# Figure 4
dataset_gap = xr.load_dataset('../data/fig_3.nc')
gaps, gaps_detuned = dataset_gap['data'].data
asyms_zoom = dataset_gap.coords[r'$\Gamma_-$'].data
phis_zoom = dataset_gap.coords[r'$\phi$'].data
detuning = dataset_gap.coords[r'$\delta\phi$'].data[1]

# %%
detun_phis = (gaps_detuned[:, :, 0] - gaps[:, :, 0])/detuning
gaps[:, :, 1][np.abs(gaps[:, :, 0])>1e-7] = np.nan
detun_phis[np.abs(gaps[:, :, 0])>1e-7] = np.nan

# %%
arg_maxphi = len(phis_zoom)//2
arg_maxtau = np.argmax(np.nan_to_num(np.abs(gaps[:, arg_maxphi, 1])))

# %%
fig, ax = plt.subplots(ncols=2, figsize=(plot_width * 0.85, 3))
fig.tight_layout(w_pad=0)

plot2 = ax[0].pcolormesh(
    2*phis_zoom/np.pi, asyms_zoom**2, np.abs(detun_phis),
    vmin=0, cmap='plasma', rasterized=True
)
plot = ax[1].pcolormesh(
    2*phis_zoom/np.pi, asyms_zoom**2, np.abs(gaps[:, :, 1]),
    cmap='viridis', rasterized=True
)

ax[0].set_ylabel(r'$\eta$')
ax[1].set_yticklabels([])
for axes in ax:
    axes.scatter(*[2*phis_zoom[arg_maxphi]/np.pi, asyms_zoom[arg_maxtau]**2], c='orange', marker='x')
    axes.set_xlabel(r'$\phi/\pi$')
    axes.set_ylim(0., 0.05)
    axes.set_xlim(0.6, 1.4)
    #axes.set_yticks([0.95, 0.98, 0.99, 1])
    axes.set_xticks([0.7, 1, 1.3])

cbar1 = plt.colorbar(plot, ax=ax[1], location='top')
cbar1.ax.set_xlabel(r'$E_\text{gap}/\Gamma_+$', labelpad=15)
cbar2 = plt.colorbar(plot2, ax=ax[0], location='top')
cbar2.ax.set_xlabel(r'$\delta E_\text{M} / \delta\phi$', labelpad=15)

ax[0].text(0.4, .067, '(a)')
ax[1].text(0.5, .067, '(b)')

# %%
plt.savefig(
    "../publication/figures/optimal.svg", bbox_inches='tight', transparent=True, dpi=500
    )

# %% [markdown]
# Figure appendix

# %%
data_wfs = xr.load_dataset('../data/fig_app.nc')
wfs, pol = data_wfs['data'].data
phi = data_wfs.coords[r'$\phi$'].data
theta = data_wfs.coords[r'$\theta$'].data
gamma_m = data_wfs.attrs['gamma_m']

# %%
y = np.pi/2 - np.arctan(np.tan(phi- np.pi/2)/gamma_m)

# %%
fig, ax = plt.subplots(ncols=2, figsize=(7, 4))

im1 = ax[0].pcolormesh(2*theta/np.pi, 2*phi/np.pi, pol, vmin=0, vmax=1, rasterized=True)
im2 = ax[1].pcolormesh(2*theta/np.pi, 2*phi/np.pi, wfs, cmap='inferno', vmin=0, vmax=0.75, rasterized=True)
cbar_1 = plt.colorbar(im1, ax=ax[0], location='top')
cbar_2 = plt.colorbar(im2, ax=ax[1], location='top')
cbar_1.ax.set_xlabel(r'$\mathcal{M}_{L}$', labelpad=15)
cbar_2.ax.set_xlabel(r'$|\psi_1(x_L)|^2$', labelpad=15)

ax[0].plot(2*y/np.pi, 2*phi/np.pi, c='black')
ax[0].set_xlabel('$\\theta/\pi$')
ax[1].set_xlabel('$\\theta/\pi$')
ax[0].set_ylabel('$\phi/\pi$')
ax[1].set_yticklabels([]);
ax[0].set_yticks([0,1,2])
ax[1].set_yticks([0,1,2])

ax[0].text(-0.4, 2.7, '(a)')
ax[1].text(-0.4, 2.7, '(b)')

# %%
plt.savefig(
    "../publication/figures/gauge.svg",
    bbox_inches='tight', transparent=False, dpi=500
    )

# %%
