import numpy as np
import numpy.linalg as nla

def finite_bias(ham, Vbias, c_dagger_m, c_m, lead_params):
    """Calculate the zero bias conductance matrix.
    
    Parameters
    ==========

    ham             : Number of sites.

    Vbias           : Voltage bias applied to the leads.

    c_dagger_m, c_m : tuple of creation and annihilation matrices.

    lead_params     : Dictionary of lead parameters.

    Returns

    =======

    I               : Four components of currents.
    
    """  
    evs, evecs = nla.eigh(ham)
    Tlist = transition_matrix(evecs, c_dagger_m, c_m)

    _lead_params = {**lead_params}
    _lead_params["VL"] = Vbias
    _lead_params["VR"] = 0
    nlist_L = occupation_prob(evs, _lead_params)
    I_left_V0, I_right_V0 = current(Tlist, nlist_L, _lead_params)

    _lead_params["VL"] = 0
    _lead_params["VR"] = Vbias
    nlist_R = occupation_prob(evs, _lead_params)
    I_left_0V, I_right_0V = current(Tlist, nlist_R, _lead_params)

    return I_left_V0, I_left_0V, I_right_V0, I_right_0V


def transition_matrix(evecs, c_dagger_m, c_m):
    N = int(len(c_dagger_m) / 2)    
    tL_up_dagger = evecs.T.conj() @ c_dagger_m[0] @ evecs
    tL_down_dagger = evecs.T.conj() @ c_dagger_m[1] @ evecs
    tR_up_dagger = evecs.T.conj() @ c_dagger_m[-2] @ evecs
    tR_down_dagger = evecs.T.conj() @ c_dagger_m[-1] @ evecs

    tL_dagger = (
        np.asarray(np.abs(tL_up_dagger)) ** 2 + np.asarray(np.abs(tL_down_dagger)) ** 2
    )
    tL = tL_dagger.T
    tR_dagger = (
        np.asarray(np.abs(tR_up_dagger)) ** 2 + np.asarray(np.abs(tR_down_dagger)) ** 2
    )

    tR = tR_dagger.T

    return [tL_dagger, tR_dagger, tL, tR]


def occupation_prob(evs, lead_params):
    VL, VR, T = lead_params["VL"], lead_params["VR"], lead_params["T"]
    energy_mesh = evs[:, None] - evs[None, :]
    nL_dagger = fermi_dist(energy_mesh, VL, T)
    nR_dagger = fermi_dist(energy_mesh, VR, T)
    nL = 1 - fermi_dist(-energy_mesh, VL, T)
    nR = 1 - fermi_dist(-energy_mesh, VR, T)

    return [nL_dagger, nR_dagger, nL, nR]


def current(Tlist, nlist, lead_params):
    WL_dagger, WR_dagger, WL, WR = gamma(Tlist, nlist, lead_params)
    W_matrix = WL_dagger + WL + WR_dagger + WR
    P_vector = probability_vector(W_matrix.astype(float)).T
    left_current = np.sum((WL_dagger.astype(float) - WL.astype(float)) @ P_vector)
    right_current = np.sum((WR_dagger.astype(float) - WR.astype(float)) @ P_vector)

    return left_current, right_current


def gamma(Tlist, nlist, lead_params):
    gammaL = lead_params["gammaL"]
    gammaR = lead_params["gammaR"]
    WL_dagger = gammaL * np.multiply(Tlist[0], nlist[0])
    WR_dagger = gammaR * np.multiply(Tlist[1], nlist[1])
    WL = gammaL * np.multiply(Tlist[2], nlist[2])
    WR = gammaR * np.multiply(Tlist[3], nlist[3])

    WL_dagger = WL_dagger - np.diag(np.diag(WL_dagger))
    WR_dagger = WR_dagger - np.diag(np.diag(WR_dagger))
    WL = WL - np.diag(np.diag(WL))
    WR = WR - np.diag(np.diag(WR))

    return WL_dagger, WR_dagger, WL, WR


def fermi_dist(E, mu, T):
    return 1.0 / (1.0 + np.exp((E - mu) / T))


def probability_vector(W_matrix):
    """
    Returns the probability vector in steady
    state, dP/dt = 0.
    """
    N_state = W_matrix.shape[0]
    for i in range(N_state):
        W_matrix[i, i] = -np.sum(W_matrix[:, i])
    W_matrix = np.vstack((W_matrix, np.ones(N_state)))

    rhs = np.append(np.zeros(N_state), 1)
    P_vector = nla.pinv(W_matrix) @ rhs

    return P_vector