from scipy.optimize import minimize
from scipy.linalg import eigvals
import numpy as np
from .second_quantization import evaluate_hamiltonian
from .constants import ham_terms

def even_odd_splitting(mu, params):
    """
    Calculates the difference between the lowest energy of 
    the even and odd subspaces
    """
    params['muL'] = mu
    params['muR'] = mu
    pmm_even, pmm_odd = evaluate_hamiltonian(ham_terms, params)
    even_evals = np.sort(eigvals(pmm_even))
    odd_evals = np.sort(eigvals(pmm_odd))
    return - even_evals[0] + odd_evals[0]


def wrapped_minimize(value, params, key):
    """
    Minimizer that returns the absolute value of the difference 
    between the lowest energy of the even and odd subspaces as a
    function of any parameter in the Hamiltonian.
    """
    params[key] = value
    result = minimize(
        even_odd_splitting,
        x0=0,
        args=params,
        bounds=[(0, 1),], # bounds for muL=muR
    )

    return abs(result.fun)


def wrapped_minimize_LR(mu, params):
    """
    Minimizer for mu_L =mu_R
    """
    return even_odd_splitting(mu, params)


def run_minimization(params, key, bounds):
    """
    Find the sweet spot for mu_L=mu_R and an arbitrary parameter
    defined by `key` and `bounds`.
    """
    # this will give the optimal phase
    result_M = minimize(
        wrapped_minimize,
        x0=(0.15),
        bounds=bounds,
        args=(params, key)
    )
    params[key] = result_M.x[0]
    # at the optimal phase, we optimize mu_L=mu_R
    result_LR = minimize(
        wrapped_minimize_LR,
        x0=0,
        args=params,
        bounds=[(-0.5, 2),],
    )

    return result_M, result_LR