import numpy as np
import itertools


def get_h_2(c_labels, c_signs, strength, index_from_basis):
    """
    Constructs the second-order Hamiltonian from two fermionic creation/annihilation operators.

    Parameters:
        c_labels (List[int]): a list containing the labels of the two fermionic operators
        c_signs (List[int]): a list containing the signs of the two fermionic operators (+1 for creation, -1 for annihilation)
        strength (float): the strength of the interaction between the two operators
        index_from_basis (Dict[Tuple[int], int]): a dictionary mapping a basis state in binary representation to its index in `indices`

    Returns:
        H (ndarray): a complex-valued matrix representing the Hamiltonian

    When c_labels = [i, j] and c_signs = [+1, -1], it means H = strength * c^dg_i c_j (+ H.c. if needed).
    """

    indices = list(index_from_basis.values())
    bases = list(index_from_basis.keys())

    N = len(indices)
    H = np.zeros((N, N), dtype=complex)
    if c_labels[0] > c_labels[1]:  # we first apply c operators with a larger c_label
        cflip = 1
    else:
        cflip = 0

    for n in indices:
        restart = False
        num_of_anticom = 0
        vec = list(bases[n])
        num_of_anticom += np.sum(vec[0 : c_labels[0]]) + np.sum(vec[0 : c_labels[1]])
        vec[c_labels[1]] += c_signs[1]
        if any(k < 0 for k in vec) or any(k > 1 for k in vec):
            restart = True
        vec[c_labels[0]] += c_signs[0]
        if any(k < 0 for k in vec) or any(k > 1 for k in vec):
            restart = True
        if restart:
            continue

        m = index_from_basis[tuple(vec)]
        H[m, n] = strength * (-1) ** (num_of_anticom + cflip)
        if abs(m - n) > 1e-5:
            H[n, m] = H[m, n].conj()
    return H


def get_h_1(c_label, c_sign, strength, n_labels):
    """
    Generate the matrix representation of single creation and annihilation ops.
    This matrix connects even and odd spaces.
    """

    even_basis, odd_basis = organize_data(n_labels=n_labels)
    even_basis = list(list(even_basis.keys()))
    odd_basis = list(list(odd_basis.keys()))

    N = len(even_basis)
    H = np.zeros((N, N), dtype=complex)
    odd_indices = [list(indices) for indices in odd_basis]
        
    for i, basis in enumerate(even_basis):
        reset = False
        vec = list(basis)
        vec[c_label] += c_sign
        if vec[c_label] < 0 or vec[c_label] > 1:
            reset = True
        if not reset:
            j = odd_indices.index(vec)
            H[i, j] = strength
    return H




def get_h_coulomb(n_labels, strength, indices_bases_dict):
    """
    We only consider coulomb terms like U * n_i * n_j
    When n_labels = [i, j] , it means H = strength * n_i * n_j.
    """
    N = len(indices_bases_dict)
    H = np.zeros((N, N), dtype=complex)
    for n, bases in enumerate(indices_bases_dict.keys()):
        vec = list(bases)
        n_i, n_j = vec[n_labels[0]], vec[n_labels[1]]
        H[n, n] = strength * n_i * n_j
    return H

def get_c_matrix(c_labels, c_signs, bases, index_from_basis):
    """
    When c_labels = [i] and c_signs = [+1], it means <basis|c^dg_i|basis> (no h.c.).
    When c_labels = [j] and c_signs = [-1], it means <basis|c_j|basis> (no h.c.).
    """
    dim = len(bases)
    c_matrix = np.zeros((dim, dim), dtype=complex)
    
    for n in range(dim):
        restart = False
        num_of_anticom = 0
        vec = list(bases[n])
        num_of_anticom += np.sum(vec[0:c_labels[0]])
        
        vec[c_labels[0]] += c_signs[0]
        if any(k < 0 for k in vec) or any(k > 1 for k in vec):
            restart = True
        if restart:
            continue

        m = index_from_basis[tuple(vec)]
        c_matrix[m, n] = (-1)**(num_of_anticom)
    return c_matrix


def separate_bases_by_parity(all_bases):
    """
    Separates bases into even and odd parity based on the sum of their elements.
    """
    even_bases, odd_bases = [], []
    for basis in all_bases:
        if np.sum(basis) % 2 == 0:
            even_bases.append(basis)
        else:
            odd_bases.append(basis)
    return even_bases, odd_bases


def create_index_dict(bases):
    index_from_basis = {}
    for n in range(len(bases)):
        index_from_basis[bases[n]] = n
    return index_from_basis


def organize_data(n_labels):
    """
    Organizes the bases into even and odd parity and creates dictionaries that map bases to indices.
    """
    all_bases = list(itertools.product([0, 1], repeat=n_labels))
    even_bases, odd_bases = separate_bases_by_parity(all_bases)

    even_index_from_basis = create_index_dict(even_bases)
    odd_index_from_basis = create_index_dict(odd_bases)

    return even_index_from_basis, odd_index_from_basis


def make_hamiltonian(h_2_list, even_index_from_basis, odd_index_from_basis):
    """
    Create a Hamiltonian given a list of operators in second quantization.
    It contains c_labels = [i, j] and c_signs = [+1, -1], it means H = strength * c^dg_i c_j (+ H.c. if needed).
    Returns the even and odd sectors separately.
    """
    N_even = len(even_index_from_basis)
    N_odd = len(odd_index_from_basis)
    h_even_const = np.zeros((N_even, N_even), dtype=complex)
    h_odd_const = np.zeros((N_odd, N_odd), dtype=complex)

    for h_2_params in h_2_list:
        if len(h_2_params) == 3:
            h_even_const += get_h_2(*h_2_params, even_index_from_basis)
            h_odd_const += get_h_2(*h_2_params, odd_index_from_basis)
        elif len(h_2_params) == 2:
            h_even_const += get_h_coulomb(*h_2_params, even_index_from_basis)
            h_odd_const += get_h_coulomb(*h_2_params, odd_index_from_basis)

    return h_even_const, h_odd_const


def linear_terms(hamiltonian_terms, n_labels):
    """
    Extract a numpy array for each term in `hamiltonian_terms` in a space of size `2**n_labels`
    """
    even_index_from_basis, odd_index_from_basis = organize_data(n_labels)

    matrices = {}
    for key, terms in hamiltonian_terms.items():
        matrices[key] = make_hamiltonian(
            terms, even_index_from_basis, odd_index_from_basis
        )
    return matrices


def evaluate_hamiltonian(terms, params, permute=False):
    """
    Compute the Hamiltonian from `term` for a set of parameters
    """
    permutation = [0, 4, 3, 7, 1, 2, 5, 6]
    example_matrix = list(terms.values())[0][0]  
    n = example_matrix.shape[0]
    h_even = np.zeros((n, n), dtype=complex)
    h_odd = np.zeros((n, n), dtype=complex)
    for key, value in terms.items():
        h_even += value[0] * params[key]
        h_odd += value[1] * params[key]
    if permute:
        h_even = h_even[:, permutation] 
        h_even = h_even[permutation, :] 
        h_odd = h_odd[:, permutation] 
        h_odd = h_odd[permutation, :] 
    return h_even, h_odd


def single_operators(n_labels):
    dim_fock_space = 2**n_labels
    unsorted_bases = list(itertools.product([0, 1], repeat=n_labels))
    
    even_bases = []
    odd_bases = []
    
    for vec in unsorted_bases:
        if np.sum(vec) % 2 == 0:
            even_bases.append(vec)
        else:
            odd_bases.append(vec)
    
    bases = even_bases + odd_bases
    index_from_basis = {}
    for n in range(dim_fock_space):
        index_from_basis[bases[n]] = n
    
    cs = []
    csd = []
    for i in range(n_labels):
        cs.append(get_c_matrix([i], [-1], bases, index_from_basis))
        csd.append(get_c_matrix([i], [+1], bases, index_from_basis))
        
    return cs, csd


def majoran_operators(phi, ops, ops_dagger):
    ws = []
    zs = []
    for i in range(len(ops)):
        ws.append(np.exp(1j*phi) * ops[i] + np.exp(-1j*phi)*ops_dagger[i])
        zs.append(np.exp(1j*phi) * ops[i] - np.exp(-1j*phi)*ops_dagger[i])

    ws = np.array(ws)
    zs = np.array(zs)
    return ws, zs 